from time import sleep
from math import pi, cos, sin
import numpy as np
from numpy.linalg import pinv
from ctypes import *
from numpy.random import uniform
lib = cdll.Project1

class EMTracker(object):
    def testfun(self):
        return 5
    
    def __init__(self):
        self.obj = lib.EMTracker_new()
		
    def setup(self):
        lib.setup(self.obj)
		
    def read(self):
        lib.read(self.obj)
		
    def turnOff(self):
        lib.turnOff(self.obj)
		
    def getX(self):
        return lib.getX(self.obj)
		
    def getY(self):
        return lib.getY(self.obj)

    def getZ(self):
        return lib.getZ(self.obj)
	
    def getAzimuth(self):
        return lib.getAzimuth(self.obj)
		
    def getElevation(self):
        return lib.getElevation(self.obj)
		
    def getRoll(self):
        return lib.getRoll(self.obj)
        
def pivot_calibration( Rs, ps ):
    n = len( Rs )
    A = np.hstack( [np.vstack(Rs), np.vstack( [-np.eye(3)]*n )] )
    b = np.vstack(ps)
    return pinv(A).dot(-b)
    
#make a rotation matrix with a given azimuth, elevation, and roll
def make_rotation(a, e, r):
    Z = np.array([[cos(a), -sin(a), 0],[sin(a), cos(a),0],[0,0,1]])
    Y = np.array([[cos(e), 0, sin(e)],[0,1,0],[-sin(e),0,cos(e)]])
    X = np.array([[1,0,0],[0,cos(r),-sin(r)],[0,sin(r),cos(r)]])
    return X.dot(Y).dot(Z);

def random_transform(p, p_):
    a,e,r = uniform(0,2*pi,3)
    R = make_rotation(a,e,r)
    t = R.dot(p) - p_
    return R,t
    
lib.getX.restype = c_double;
lib.getY.restype = c_double;
lib.getZ.restype = c_double;
lib.getAzimuth.restype = c_double;
lib.getElevation.restype = c_double;
lib.getRoll.restype = c_double;

simulation = True 
if not simulation:
    em = EMTracker()
    em.setup();

p = uniform(-100,100,[3,1])
p_ = uniform(-20,20,[3,1])

while True:
    Rs = []
    ps = []
    i = 0
    while True:
        usr = raw_input("Hit enter to take reading, write Q to do pointer calibration: ");
        if usr == "Q":
            break
        if simulation:
            R,t = random_transform(p,p_)
            Rs.append( R )
            ps.append( -1*t )
        else:
            em.read();
            x,y,z = em.getX(), em.getY(), em.getZ()
            a,e,r = em.getAzimuth()*pi/180, em.getElevation()*pi/180, em.getRoll()*pi/180
            Rs.append( make_rotation(a,e,r) )
            ps.append( -1*np.array([[x],[y],[z]]) )
        i=i+1
        print i
     
    #do pivot calibration
    x = pivot_calibration( Rs, ps ) 
    #last 3 elements of x are the pointer tip in EM tracker coordinates
    hydrophone = x[3:6]
    print x
